import qrcode
import cv2
import numpy as np
import time
from froxiaofeidriver.myvideo import Video

class QRcode(Video):
    code_path = "/home/pi/AiCar/file/"
    # cap = cv2.VideoCapture(0)
    QR_detector = cv2.QRCodeDetector()
#     flag_now = 'No'
    flag_pass = 'No'
    img1 = 0
    count = 0
    width = 320
    height = 240
    dim = (width,height) #压缩以后的视频尺寸，避免视频帧过大造成显示卡顿
    
    def __init__(self):
        pass
    def create_QRcode(self,s):
        file_img = self.code_path + s +'.png'
        img = qrcode.make(s)
        img.save(file_img)
        
        img = cv2.imread(file_img)
        cv2.putText(img,s,(0,20),cv2.FONT_HERSHEY_COMPLEX,0.5,(0,0,0),1)
        cv2.imwrite(file_img,img)
        
    def detect_QRcode(self):
        
        while True:
            success,img0 = self.cap.read()
            self.img1 = cv2.resize(img0, self.dim, interpolation = cv2.INTER_AREA)#视频帧压缩
            if success:
                flag_now,AA,BB = self.QR_detector.detectAndDecode(self.img1)
                if flag_now:
                    if flag_now == self.flag_pass:
                        self.count += 1
                        if self.count == 5:
                            self.count = 0
                            return flag_now
                        
                    self.flag_pass = flag_now
                else:
                    self.count = 0
                    return flag_now
    def QRcode_num(self,num):
        return int(num)
    
                
    def cv2_show(self,name):
        
        cv2.imshow(name,self.img1)
        cv2.waitKey(1)

    def cv2_destroy(self):
        cv2.destroyAllWindows()
                    
                    
            
            
            
            
    


from .mycarmotor import Carmotor
from .mypygame import Mypygame
from .myserial import Myserial
from .myservo import Servo
from .mysound import Sound
import froxiaofeidriver.myvideo
from .mygesture import Guester
from .myface import Face
from .mycolor import Color
from .mytts import Pyttsx
from .mytrafficsign import TrafficSign
from .myQRcode import QRcode
from .mycarlicense import License



import pigpio


"""
@树莓派上面四路电机的速度PWM值控制
"""
class Carmotor(object):
    def __init__(self,in1 = 26,in2 = 18,in3 = 7,in4 = 8,in5 = 12,in6 = 13,in7 = 20,in8 = 16): #这是BCM编码序号，分别对应树莓派扩展接口的P37、P12、P26、P24，四个GPIO连接到L298P芯片
        self.Pi = pigpio.pi()
        self.In1 = in1
        self.In2 = in2
        self.In3 = in3
        self.In4 = in4
        self.In5 = in5
        self.In6 = in6
        self.In7 = in7
        self.In8 = in8

        self.Pi.set_PWM_range(self.In1,100) #pwm范围
        self.Pi.set_PWM_range(self.In2,100) #pwm范围
        self.Pi.set_PWM_range(self.In3,100) #pwm范围
        self.Pi.set_PWM_range(self.In4,100) #pwm范围
        self.Pi.set_PWM_range(self.In5,100) #pwm范围
        self.Pi.set_PWM_range(self.In6,100) #pwm范围
        self.Pi.set_PWM_range(self.In7,100) #pwm范围
        self.Pi.set_PWM_range(self.In8,100) #pwm范围

        self.Pi.set_PWM_frequency(self.In1,10000) #频率10Khz
        self.Pi.set_PWM_frequency(self.In2,10000)
        self.Pi.set_PWM_frequency(self.In3,10000)
        self.Pi.set_PWM_frequency(self.In4,10000)
        self.Pi.set_PWM_frequency(self.In5,10000) #频率10Khz
        self.Pi.set_PWM_frequency(self.In6,10000)
        self.Pi.set_PWM_frequency(self.In7,10000)
        self.Pi.set_PWM_frequency(self.In8,10000)
        

        self.Pi.set_PWM_dutycycle(self.In1,0) #暂停PWM输出
        self.Pi.set_PWM_dutycycle(self.In2,0)
        self.Pi.set_PWM_dutycycle(self.In3,0)
        self.Pi.set_PWM_dutycycle(self.In4,0)
        self.Pi.set_PWM_dutycycle(self.In5,0) #暂停PWM输出
        self.Pi.set_PWM_dutycycle(self.In6,0)
        self.Pi.set_PWM_dutycycle(self.In7,0)
        self.Pi.set_PWM_dutycycle(self.In8,0)

    """
    @设置四个电机的速度PWM值大小
    """
    def set_speed(self, Front_Left, Front_Right, Back_Left, Back_Right):
        
        Front_Left  = -100 if Front_Left < -100  else Front_Left #超出范围按边界值设置
        Front_Left  = 100  if Front_Left > 100   else Front_Left
        Front_Right = 100  if Front_Right > 100  else Front_Right
        Front_Right = -100 if Front_Right < -100 else Front_Right
        
        Back_Left  = -100 if Back_Left < -100  else Back_Left #超出范围按边界值设置
        Back_Left  = 100  if Back_Left > 100   else Back_Left
        Back_Right = 100  if Back_Right > 100  else Back_Right
        Back_Right = -100 if Back_Right < -100 else Back_Right
        

        DutyIn1 = 0 if Front_Left < 0 else Front_Left
        DutyIn2 = 0 if Front_Left > 0 else -Front_Left
        DutyIn3 = 0 if Front_Right < 0 else Front_Right
        DutyIn4 = 0 if Front_Right > 0 else -Front_Right
        
        DutyIn5 = 0 if Back_Left < 0 else Back_Left
        DutyIn6 = 0 if Back_Left > 0 else -Back_Left
        DutyIn7 = 0 if Back_Right < 0 else Back_Right
        DutyIn8 = 0 if Back_Right > 0 else -Back_Right

        self.Pi.set_PWM_dutycycle(self.In1,DutyIn1) #设置PWM输出的占空比
        self.Pi.set_PWM_dutycycle(self.In2,DutyIn2)
        self.Pi.set_PWM_dutycycle(self.In3,DutyIn3)
        self.Pi.set_PWM_dutycycle(self.In4,DutyIn4)
        
        self.Pi.set_PWM_dutycycle(self.In5,DutyIn5) #设置PWM输出的占空比
        self.Pi.set_PWM_dutycycle(self.In6,DutyIn6)
        self.Pi.set_PWM_dutycycle(self.In7,DutyIn7)
        self.Pi.set_PWM_dutycycle(self.In8,DutyIn8)

    """
    @四个电机停止
    """
    def stop(self):
        self.set_speed(0,0,0,0)

    """
    @设置前进
    @src：速度PWM值取值范围40~~100，一般启动速度PWM值得大于65
    """
    def go_ahead(self,src):
        if (src >39) and (src < 101):
            self.set_speed(-src,src,-src,src)
        else:
            print('\033[31m warning: speed value must be 40,41...100 \033[0m')

    """
    @设置后退
    @src：速度PWM值取值范围40~~100，一般启动速度PWM值得大于65
    """
    def go_back(self,src):
        if (src >39) and (src < 101):
            self.set_speed(src,-src,src,-src)
        else:
            print('\033[31m warning: speed value must be 40,41...100 \033[0m')

    """
    @设置向左转
    @src：速度PWM值取值范围40~~100，一般启动速度PWM值得大于65
    """
    def turn_left(self,src):
        if (src >39) and (src < 101):
            self.set_speed(src,src,src,src)
        else:
            print('\033[31m warning: speed value must be 40,41...100 \033[0m')

    """
    @设置向右转
    @src：速度PWM值取值范围40~~100，一般启动速度PWM值得大于65
    """
    def turn_right(self,src):
        if (src >39) and (src < 101):
            self.set_speed(-src,-src,-src,-src)
        else:
            print('\033[31m warning: speed value must be 40,41...100 \033[0m')

    """
    @设置左漂移
    @src：速度PWM值取值范围40~~100，一般启动速度PWM值得大于65
    """
    def shift_left(self,src):
        if (src >39) and (src < 101):
            self.set_speed(src,src,-src,-src)
        else:
            print('\033[31m warning: speed value must be 40,41...100 \033[0m')

    """
    @设置右漂移
    @src：速度PWM值取值范围40~~100，一般启动速度PWM值得大于65
    """
    def shift_right(self,src):
        if (src >39) and (src < 101):
            self.set_speed(-src,-src,src,src)
        else:
            print('\033[31m warning: speed value must be 40,41...100 \033[0m')

    """
    @设置向左中心轴转
    @src：速度PWM值取值范围40~~100，一般启动速度PWM值得大于65
    """
    def circle_left(self,src):
        if (src >39) and (src < 101):
            self.set_speed(src,src,0,0)
        else:
            print('\033[31m warning: speed value must be 40,41...100 \033[0m')

    """
    @设置向右中心轴转
    @src：速度PWM值取值范围40~~100，一般启动速度PWM值得大于65
    """
    def circle_right(self,src):
        if (src >39) and (src < 101):
            self.set_speed(-src,-src,0,0)
        else:
            print('\033[31m warning: speed value must be 40,41...100 \033[0m')
        



import cv2
import time
import numpy as np
import mediapipe as mp
from PIL import Image,ImageFont,ImageDraw
from flask import Flask,render_template,Response,request,jsonify,redirect,url_for
from froxiaofeidriver.myvideo import Video

class Guester(Video):
    
    dic_guester={1:"识别结果:1",2:"识别结果:2",3:"识别结果:3",4:"识别结果:4",5:"识别结果:5",6:"识别结果:6",8:"识别结果:8",0:"识别结果:0"}
        
    # cap = cv2.VideoCapture(0)
    mpHands = mp.solutions.hands
    hands = mpHands.Hands()
    mpDraw = mp.solutions.drawing_utils
    c_guester = None
    p_guester = None
    str_show = ""
    img1 = 0
    
    def __init__(self):
#         super(Guester,self).__init__()
        pass
    
    def get_guester(self,img,list_lms):
        hull_index = [0,1,2,3,6,10,14,19,18,17]
        up_hull = [4,8,12,16,20]
        up_fingers = []
        guester = None
        
        hull = cv2.convexHull(list_lms[hull_index,:])
        cv2.polylines(img,[hull],True,(0,255,0),2)
        
        for i in up_hull:
            pt = (int(list_lms[i][0]),int(list_lms[i][1]))
            dist = cv2.pointPolygonTest(hull,pt,True)
            if dist < 0:
                up_fingers.append(i)
        
        if len(up_fingers) == 1 and up_fingers[0] == 8:
            guester = 1
        elif len(up_fingers) == 2 and up_fingers[0] == 8 and up_fingers[1] == 12:
            guester = 2
        elif len(up_fingers) == 3 and up_fingers[0] == 8 and up_fingers[1] == 12 and up_fingers[2] == 16:
            guester = 3
        elif len(up_fingers) == 4 and up_fingers[0] == 8 and up_fingers[1] == 12 and up_fingers[2] == 16 and up_fingers[3] == 20:
            guester = 4
        elif len(up_fingers) == 5:
            guester = 5
        elif len(up_fingers) == 2 and up_fingers[0] == 4 and up_fingers[1] == 20:
            guester = 6
        elif len(up_fingers) == 2 and up_fingers[0] == 4 and up_fingers[1] == 8:
            guester = 8
        elif len(up_fingers) == 0:
            guester = 0
         
        return guester
         
    def paint_chinese_opencv(self,im,chinese,pos,color,font_size=20):
        img_PIL = Image.fromarray(cv2.cvtColor(im,cv2.COLOR_BGR2RGB))
        font = ImageFont.truetype('/home/pi/AiCar/mylib/NotoSansCJK-Bold.otf',font_size,encoding="utf-8")
        fillColor = color
        position =pos
        
        draw = ImageDraw.Draw(img_PIL)
        draw.text(position,chinese,fillColor,font)
        img = cv2.cvtColor(np.asarray(img_PIL),cv2.COLOR_RGB2BGR)
        
        return img
    
    def get_gesture_img(self):
        
        while True:
            success,self.img1 = self.cap.read()
            if not success:
                continue
            image_height,image_width,_ = np.shape(self.img1)
            imgRGB = cv2.cvtColor(self.img1,cv2.COLOR_BGR2RGB)
            results = self.hands.process(imgRGB)
            
            if results.multi_hand_landmarks:
                hand = results.multi_hand_landmarks[0]
                self.mpDraw.draw_landmarks(self.img1,hand,self.mpHands.HAND_CONNECTIONS)
                list_lms = []
                for i in range(21):
                    pos_x = hand.landmark[i].x*image_width
                    pos_y = hand.landmark[i].y*image_height
                    list_lms.append([int(pos_x),int(pos_y)])
                
                list_lms = np.array(list_lms,dtype = np.int32)
                self.c_guester = self.get_guester(self.img1,list_lms)
                
                if not self.c_guester is None and self.c_guester != self.p_guester:
                    self.p_guester = self.c_guester
            if not self.p_guester is None:
                self.str_show = ' %s'%(self.dic_guester[self.p_guester])
            self.img1 = self.paint_chinese_opencv(self.img1,self.str_show,(10,10),(0,255,255),font_size=30)

        
            return self.p_guester
        cap.release()
         
    def gesture_web_video(self):
        
        while True:
            success,img = self.cap.read()
            if not success:
                continue
            image_height,image_width,_ = np.shape(img)
            imgRGB = cv2.cvtColor(img,cv2.COLOR_BGR2RGB)
            results = self.hands.process(imgRGB)
            
            if results.multi_hand_landmarks:
                hand = results.multi_hand_landmarks[0]
                self.mpDraw.draw_landmarks(img,hand,self.mpHands.HAND_CONNECTIONS)
                list_lms = []
                for i in range(21):
                    pos_x = hand.landmark[i].x*image_width
                    pos_y = hand.landmark[i].y*image_height
                    list_lms.append([int(pos_x),int(pos_y)])
                
                list_lms = np.array(list_lms,dtype = np.int32)
                self.c_guester = self.get_guester(img,list_lms)
                
                if not self.c_guester is None and self.c_guester != self.p_guester:
                    self.p_guester = self.c_guester
            if not self.p_guester is None:
                self.str_show = ' %s'%(self.dic_guester[self.p_guester])
            img = self.paint_chinese_opencv(img,self.str_show,(10,10),(0,255,255),font_size=30)

            ret,jpeg = cv2.imencode('.jpg',img)
            yield(b'--frame\r\n'+b'Content-Type:image/jpeg\r\n\r\n'+jpeg.tobytes()+b'\r\n\r\n')
        cap.release()
        
    def return_gesture(self):
        
        return self.p_guester
    
    def run(self):
         
        while True:
            gesture_flag, img = self.get_gesture_img() 
            ret,jpeg = cv2.imencode('.jpg',img)
            yield(b'--frame\r\n'+b'Content-Type:image/jpeg\r\n\r\n'+jpeg.tobytes()+b'\r\n\r\n')
    
    def cv2_show(self,name):
        cv2.imshow(name,self.img1)
        cv2.waitKey(1)

    def cv2_destroy(self):
        cv2.destroyAllWindows()
        
    def img_encode(self,img):
        ret,jpeg = cv2.imencode('.jpg',img)
        return jpeg
                
            